with win32.objbase; use win32.objbase;
with interfaces.c; use interfaces.c;

package Factory is

   subtype HRESULT is unsigned_long;

   type IClassFactory;
   -- Class factory object forward reference

   type IClassFactory_Pointer is access all IClassFactory;
   -- Pointer to Class Factory Object


   -- Access function types for IUnknown part of IClassFactory

   type af_QueryInterface is access function (This      : access IClassFactory;
                                              riid      : REFIID;
                                              ppvObject : access Win32.PVOID)
     return HRESULT;
   pragma Convention(Stdcall, af_QueryInterface);

   type af_AddRef is access function (This : access IClassFactory)
     return Win32.ULONG;
   pragma Convention(Stdcall, af_AddRef);

   type af_Release is access function (This : access IClassFactory)
     return Win32.ULONG;
   pragma Convention(Stdcall, af_Release);

   -- Access function types for IClassFactory

   type af_CreateInstance is access function (This      : access IClassFactory;
                                              pUnkOuter : LPUNKNOWN;
                                              riid      : REFIID;
                                              ppvObject : access Win32.PVOID)
     return HRESULT;
   pragma Convention(Stdcall, af_CreateInstance);

   type af_LockServer is access function (This  : access IClassFactory;
                                          fLock : Win32.BOOL)
    return HRESULT;
   pragma Convention(Stdcall, af_LockServer);


   -- Function specs for IClassFactory
   --
   -- The first three functions QueryInterface, Addref and Release are from
   -- IClassFactory's parent interface, IUnknown. This allows every interface
   -- to be treated polymorphically as IUnknown. Then QueryInterface can be
   -- used to query the object for support of an interface and retrieve a
   -- pointer to the COM object's interface that conforms to that interface.
   -- In essence QueryInterface is used to do a typesafe "cast" from one
   -- interface of a COM object to another.


   function QueryInterface  (This      : access IClassFactory;
                             riid      : REFIID;
                             ppvObject : access Win32.PVOID)
                             return HRESULT;
   pragma Convention(Stdcall, QueryInterface);
   -- Query COM object for the interface riid and return a pointer to it
   -- if possible. QueryInterface will call AddRef on the interface before
   -- returning it in ppvObject

   function AddRef (This : access IClassFactory)
                    return Win32.ULONG;
   pragma Convention(Stdcall, AddRef);
   -- Add a reference count to the object

   function Release (This : access IClassFactory)
                     return Win32.ULONG;
   pragma Convention(Stdcall, Release);
   -- Reduce reference count. If 0 free object from memory

   function CreateInstance (This      : access IClassFactory;
                            pUnkOuter : LPUNKNOWN;
                            riid      : REFIID;
                            ppvObject : access Win32.PVOID)
                            return HRESULT;
   pragma Convention(Stdcall, CreateInstance);
   -- Create an instance of the COM object returning a pointer to an interface
   -- with the IID riid.
   -- Note that pUnkOuter can not be an access parameter since pUnkOuter
   -- may be null

   function LockServer (This  : access IClassFactory;
                        fLock : Win32.BOOL)
                        return HRESULT;
   pragma Convention(Stdcall, LockServer);
   -- if fLock is true add a lock to prevent the host server from unloading
   -- from memory.

   type IClassFactory_Vtbl_Record is
      record
         QueryInterface : af_QueryInterface := Factory.QueryInterface'Access;
         AddRef         : af_AddRef := Factory.AddRef'Access;
         Release        : af_Release := Factory.Release'Access;
         CreateInstance : af_CreateInstance := Factory.CreateInstance'Access;
         LockServer     : af_LockServer := Factory.LockServer'Access;
      end record;
   pragma Convention(C, IClassFactory_Vtbl_Record);
   type IClassFactory_Vtbl_Pointer is access all IClassFactory_Vtbl_Record;
   -- IClassFactory table of functions

   IClassFactory_Vtbl : aliased IClassFactory_Vtbl_Record;
   -- Create an image of the IClassFactory table in memory

   type IClassFactory is
      record
         Vtbl      : IClassFactory_Vtbl_Pointer := IClassFactory_Vtbl'Access;
         Ref_Count : aliased Win32.LONG := 1;
      end record;
   pragma Convention(C, IClassFactory);
   -- Create the IClassFactory interface
   -- Since the Class Factory Object only contains a single inheritance chain
   -- of interfaces, ie. IUnknown <-- IClassFactory. We just treat the
   -- IClassFactory Interface as the Object and its Ref_Count as the Object
   -- wide reference count to simplify the code.
end Factory;
